// vim: set tw=80 ts=2 sw=2 sts=2 :
import { Component } from '@angular/core';
import { MdDialog } from '@angular/material';
import { Observable } from 'rxjs/Rx';

import { Logger } from '@nsalaun/ng-logger';
import { ToastrService } from 'ngx-toastr';

import { Account } from './account';
import { AccountBalances } from './accountBalances';
import { AccountDataSource } from './account.dataSource';
import { AccountService } from './account.service';
import { AccountDeleteModalComponent } from './accountDeleteModal.component';
import { AccountEditModalComponent } from './accountEditModal.component';

@Component({
  selector: 'account-list',
  template: `
<div class="containerX">
  <div class="container">
    <button md-fab color="primary" (click)="add()">
      <md-icon>add</md-icon>
    </button>
  </div>

  <div class="container">
    <md-table #table [dataSource]="accounts">
      <ng-container mdColumnDef="name">
        <md-header-cell *mdHeaderCellDef>Nom du compte</md-header-cell>
        <md-cell *mdCellDef="let account">
          <a [routerLink]="['/account', account.id, 'operations']">
            {{ account.name }}
          </a>
        </md-cell>
      </ng-container>

      <ng-container mdColumnDef="current">
        <md-header-cell *mdHeaderCellDef>Solde courant</md-header-cell>
        <md-cell *mdCellDef="let account">
          <span
            [class.warning]="account.authorized_overdraft < 0 && account.balances?.current < 0"
            [class.error]="account.balances?.current < account.authorized_overdraft">
            {{ account.balances?.current | currency:"EUR":true }}
          </span>
        </md-cell>
      </ng-container>

      <ng-container mdColumnDef="pointed">
        <md-header-cell *mdHeaderCellDef>Solde pointé</md-header-cell>
        <md-cell *mdCellDef="let account">
          <span
            [class.warning]="account.authorized_overdraft < 0 && account.balances?.pointed < 0"
            [class.error]="account.balances?.pointed < account.authorized_overdraft">
            {{ account.balances?.pointed | currency:"EUR":true }}
          </span>
        </md-cell>
      </ng-container>

      <ng-container mdColumnDef="authorizedOverdraft">
        <md-header-cell *mdHeaderCellDef>Découvert autorisé</md-header-cell>
        <md-cell *mdCellDef="let account">
          {{ account.authorized_overdraft | currency:"EUR":true }}
        </md-cell>
      </ng-container>

      <ng-container mdColumnDef="actions">
        <md-header-cell *mdHeaderCellDef>Actions</md-header-cell>
        <md-cell *mdCellDef="let account">
          <!-- Edit account. -->
          <button md-mini-fab color="primary"
            (click)="modify(account)">
            <md-icon>mode_edit</md-icon>
          </button>

          <!-- Delete account, with confirm. -->
          <button md-mini-fab color="warn"
            (click)="confirmDelete(account)">
            <md-icon>delete_forever</md-icon>
          </button>

          <!-- Open account scheduler. -->
          <button md-mini-fab
            [hidden]="!account.id"
            [routerLink]="['/account', account.id, 'scheduler']">
            <md-icon>event</md-icon>
          </button>
        </md-cell>
      </ng-container>

      <md-header-row *mdHeaderRowDef="displayedColumns"></md-header-row>
      <md-row *mdRowDef="let row; columns: displayedColumns;">
      </md-row>
    </md-table>
  </div>
</div>
    `,
})
export class AccountListComponent {
  displayedColumns: String[] = [
    'name', 'current', 'pointed', 'authorizedOverdraft', 'actions'
  ];

  constructor(
    private accounts: AccountDataSource,
    private accountService: AccountService,
    private toastrService: ToastrService,
    private logger: Logger,
    private mdDialog: MdDialog,
  ) {
  }

  /*
   * Add an empty account.
   */
  add() {
    this.modify(new Account());
  };

  /*
   * Modify an account.
   */
  modify(account: Account) {
    let dialogRef = this.mdDialog.open(AccountEditModalComponent, {
      data: {
        account: account,
      }
    });

    dialogRef.afterClosed().subscribe((account: Account) => {
      if(account) {
        this.logger.log("Modal closed => save account", account);
        this.save(account);
      } else {
        this.logger.log("Modal dismissed");
      }
    }, (reason) => function(reason) {
    });
  }

  /*
   * Save account.
   */
  save(account) {
    this.accountService.create(account).subscribe(account => {
      this.toastrService.success('Account #' + account.id + ' saved.');
    }, result => {
      this.logger.error('Error while saving account', account, result);

      this.toastrService.error(
        'Error while saving account: ' + result.message
      );
    });
  };

  /*
   * Show a dialog to confirm account deletion.
   */
  confirmDelete(account: Account) {
    let dialogRef = this.mdDialog.open(AccountDeleteModalComponent, {
      data: {
        account: account,
      }
    });

    dialogRef.afterClosed().subscribe((account: Account) => {
      if(account) {
        this.delete(account);
      }
    }, reason => {
      this.logger.error("Delete dialog failed", reason);
    });
  };

  /*
   * Delete an account.
   */
  delete(account: Account) {
    var id = account.id;

    this.accountService.delete(account).subscribe(account => {
      this.toastrService.success('account #' + id + ' deleted.');

      // FIXME Alexis Lahouze 2017-09-17 Remove from array.
    }, function(result) {
      this.toastrService.error(
        'An error occurred while trying to delete account #' +
        id + ':<br />' + result
      );
    });
  };
};
